{% extends "layout.html" %}
{% block body %}
    <h1>Select & Upload an image for GNS3</h1>
    <form enctype="multipart/form-data" action="/upload" method="post">
    File path: <input type="file" name="file" /><br>
    File type: <select name="type" />
        <option value="IOU">IOU</option>
        <option value="IOURC">IOU licence (iourc)</option>
        <option value="IOS">IOS</option>
        <option value="QEMU">Qemu</option>
    </select>
    <br />
    <br />
    <input type="submit" value="Upload" />
    </form>
    {%if files%}
        <h2>Files on {{gns3_host}}</h2>
        {%for file in files%}
            <p>{{file}}</a></p>
        {%endfor%}
    {%endif%}
{% endblock %}
